/*!
 * drslib
 * Apache Licence 2.0
 * https://github.com/cocop/drslib/blob/master/LICENSE
 */
/**
 * Action interface
 */
export interface IAction<TParam, TResult> {
    do(param: TParam): TResult;
}
/**
 * Type parameter is indistinguishable
 * @param arg True if the instance implements IAction
 */
export declare function isIAction<TParam, TResult>(arg: any): arg is IAction<TParam, TResult>;
export declare type Syncable<T> = T | Promise<T>;
export declare type VoidSyncable = Syncable<void>;
/**
 * base class for common constructor
 */
export declare abstract class BAction<TParam, TResult, TValues> implements IAction<TParam, TResult> {
    protected $: TValues;
    constructor(values: TValues);
    abstract do(param: TParam): TResult;
}
/**
 * base class for action accept constructor
 */
export declare abstract class BOuter<TOuterParam, TOuterResult, TInnerParam, TInnerResult> implements IAction<TOuterParam, TOuterResult> {
    protected _inner: IAction<TInnerParam, TInnerResult>;
    constructor(inner: IAction<TInnerParam, TInnerResult>);
    abstract do(param: TOuterParam): TOuterResult;
}
/**
 * execute function
 */
export declare class Run<TParam, TResult> implements IAction<TParam, TResult> {
    private _func;
    constructor(func: (param: TParam) => TResult);
    do(param: TParam): TResult;
}
/**
 * execute getting function
 */
export declare class Get<TResult> implements IAction<void, TResult> {
    private _ref;
    constructor(ref: RefReader<TResult> | (() => TResult));
    do(param: void): TResult;
}
/**
 * execute setting function
 */
export declare class Set<TParam> implements IAction<TParam, void> {
    private _ref;
    constructor(ref: RefWriter<TParam> | ((p: TParam) => void));
    do(param: TParam): void;
}
/**
 * If isDo is true, execute inner
 */
export declare class IfValid<TParam, TResult> extends BOuter<TParam, TResult | void, TParam, TResult> {
    private readonly _isDo;
    constructor(isDo: IAction<void, boolean>, inner: IAction<TParam, TResult>);
    do(param: TParam): TResult | void;
}
/**
 * Actions that perform multiple actions
 */
export declare class RunActions<TParam, TResult> implements IAction<TParam, Promise<TResult[]>> {
    protected _list: IAction<TParam, Syncable<TResult>>[];
    constructor(list: IAction<TParam, Syncable<TResult>>[]);
    do(param: TParam): Promise<TResult[]>;
}
/**
 * An action that executes multiple actions asynchronously
 */
export declare class RunActionsParallel<TParam, TResult> extends RunActions<TParam, TResult> {
    do(param: TParam): Promise<TResult[]>;
}
/**
 * Actions that perform ordered actions
 */
export declare class RunActionsOrder<TParam, TResult> implements IAction<TParam, Promise<TResult>> {
    private _previous;
    private _executing;
    private _following;
    constructor(previous: IAction<TParam, VoidSyncable>, executing: IAction<TParam, Syncable<TResult>>, following: IAction<TParam, VoidSyncable>);
    do(param: TParam): Promise<TResult>;
}
declare const enum ChainedDoStatus {
    JoinAsync = 0,
    JoinSync = 1,
    PassAsync = 2,
    PassSync = 3
}
interface ChainedAction {
    doStatus: ChainedDoStatus;
    action: IAction<any, any>;
}
declare abstract class BChainLink<TBottomResult> {
    protected _chain: ChainedAction[];
    protected constructor(chain: ChainedAction[]);
    protected join<TResult>(action: IAction<TBottomResult, TResult>): any;
    protected pass(action: IAction<TBottomResult, void>): any;
    protected joinWait<TResult>(action: IAction<TBottomResult, Promise<TResult>>): any;
    protected passWait(action: IAction<TBottomResult, void>): any;
}
declare class SyncChainLink<TTopParam, TBottomResult> extends BChainLink<TBottomResult> {
    /**
     * Create action
     */
    create(): IAction<TTopParam, TBottomResult>;
    /**
     * Set an action that accepts a parameter and returns the next parameter
     * @param action An action that accepts a parameter and returns the next parameter
     */
    join<TResult>(action: IAction<TBottomResult, TResult>): SyncChainLink<TTopParam, TResult>;
    /**
     * Set an action that accepts parameters
     * @param action Action that accepts parameters
     */
    pass(action: IAction<TBottomResult, void>): SyncChainLink<TTopParam, TBottomResult>;
    /**
     * Sets an asynchronous action that accepts parameters and returns the next parameter
     *     Calling this method makes the action created asynchronous
     * @param action Asynchronous action that accepts a parameter and returns the next parameter
     */
    joinWait<TResult>(action: IAction<TBottomResult, Promise<TResult>>): AsyncChainLink<TTopParam, TResult>;
    /**
     * Set an asynchronous action that accepts parameters
     *     Calling this method makes the action created asynchronous
     * @param action Asynchronous actions that accept parameters
     */
    passWait(action: IAction<TBottomResult, void>): AsyncChainLink<TTopParam, TBottomResult>;
}
declare class AsyncChainLink<TTopParam, TBottomResult> extends BChainLink<TBottomResult> {
    /**
     * Create an asynchronous action
     */
    create(): IAction<TTopParam, Promise<TBottomResult>>;
    /**
     * Set an action that accepts parameters
     * @param action Action that accepts parameters
     */
    join<TResult>(action: IAction<TBottomResult, TResult>): AsyncChainLink<TTopParam, TResult>;
    /**
     * Set an action that accepts parameters
     * @param action Action that accepts parameters
     */
    pass(action: IAction<TBottomResult, void>): AsyncChainLink<TTopParam, TBottomResult>;
    /**
     * Sets an asynchronous action that accepts parameters and returns the next parameter
     * @param action Asynchronous action that accepts a parameter and returns the next parameter
     */
    joinWait<TResult>(action: IAction<TBottomResult, Promise<TResult>>): AsyncChainLink<TTopParam, TResult>;
    /**
     * Set an asynchronous action that accepts parameters
     * @param action Asynchronous actions that accept parameters
     */
    passWait(action: IAction<TBottomResult, void>): AsyncChainLink<TTopParam, TBottomResult>;
}
/**
 * A class that creates an action that pipelines the actions
 *     Use in method chain format
 */
export declare class Chain<TParam> extends SyncChainLink<TParam, TParam> {
    constructor();
}
/**
 * Action to retry until successful
 */
export declare class Retry extends BOuter<void, Promise<void>, void, Syncable<boolean>> {
    private readonly _retryCount;
    constructor(retryCount: number, inner: IAction<void, Syncable<boolean>>);
    do(): Promise<void>;
}
/**
 * Repeat action until it returns true
 */
export declare class Repetition<TParam> extends BOuter<TParam, Promise<void>, TParam, Syncable<boolean>> {
    do(param: TParam): Promise<void>;
}
/**
 * Read-only context reference interface
 */
export interface IRefReader<T> {
    get: () => T;
}
/**
 * Write-only context reference interface
 */
export interface IRefWriter<T> {
    set: (param: T) => void;
}
/**
 * Context reference interface
 */
export interface IRef<T> extends IRefReader<T>, IRefWriter<T> {
}
/**
 * Read-only context reference class
 */
export declare class RefReader<T> implements IRefReader<T> {
    get: () => T;
    constructor(get: () => T);
}
/**
 * Write-only context reference class
 */
export declare class RefWriter<T> implements IRefWriter<T> {
    set: (v: T) => void;
    constructor(set: (v: T) => void);
}
/**
 * Context reference class
 */
export declare class Ref<T> implements IRef<T> {
    get: () => T;
    set: (v: T) => void;
    constructor(get: () => T, set: (v: T) => void);
}
/**
 * Context reference class
 */
export declare class RefPath<T> implements IRef<T> {
    private readonly _context;
    private readonly _path;
    /**
     * Note that you can specify an invalid path
     * @param context Context instance
     * @param path Context path. Strings separated by "." or array
     */
    constructor(context: any, path: string | string[]);
    private _pathToArray;
    get(): T;
    set(value: T): void;
}
export {};
